@@ -78,7 +78,8 @@ typedef enum {
   GDK_DEBUG_INPUT	  = 1 <<10,
   GDK_DEBUG_CURSOR	  = 1 <<11,
   GDK_DEBUG_MULTIHEAD	  = 1 <<12,
-  GDK_DEBUG_XINERAMA	  = 1 <<13
+  GDK_DEBUG_XINERAMA	  = 1 <<13,
+  GDK_DEBUG_DRAW	  = 1 <<14
 } GdkDebugFlag;
 
 #ifndef GDK_DISABLE_DEPRECATED
@@ -219,7 +220,8 @@ GdkGC *_gdk_drawable_get_scratch_gc (GdkDrawable *drawable,
 void _gdk_gc_update_context (GdkGC     *gc,
 			     cairo_t   *cr,
 			     GdkColor  *override_foreground,
-			     GdkBitmap *override_stipple);
+			     GdkBitmap *override_stipple,
+			     gboolean   gc_changed);
 
 /*************************************
  * Interfaces used by windowing code *
@@ -259,7 +261,7 @@ void _gdk_cursor_destroy (GdkCursor *cursor);
 
 void     _gdk_windowing_init                    (void);
 
-extern GOptionEntry _gdk_windowing_args[];
+extern const GOptionEntry _gdk_windowing_args[];
 void     _gdk_windowing_set_default_display     (GdkDisplay *display);
 
 gchar *_gdk_windowing_substitute_screen_number (const gchar *display_name,
@@ -335,6 +337,32 @@ void _gdk_windowing_window_destroy_foreign (GdkWindow *window);
 void _gdk_windowing_display_set_sm_client_id (GdkDisplay  *display,
 					      const gchar *sm_client_id);
 
+#define GDK_TYPE_PAINTABLE            (_gdk_paintable_get_type ())
+#define GDK_PAINTABLE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GDK_TYPE_PAINTABLE, GdkPaintable))
+#define GDK_IS_PAINTABLE(obj)	      (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GDK_TYPE_PAINTABLE))
+#define GDK_PAINTABLE_GET_IFACE(obj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GDK_TYPE_PAINTABLE, GdkPaintableIface))
+
+typedef struct _GdkPaintable        GdkPaintable;
+typedef struct _GdkPaintableIface   GdkPaintableIface;
+
+struct _GdkPaintableIface
+{
+  GTypeInterface g_iface;
+  
+  void (* begin_paint_region) (GdkPaintable *paintable,
+			       GdkRegion    *region);
+  void (* end_paint)          (GdkPaintable *paintable);
+
+  void (* invalidate_maybe_recurse) (GdkPaintable *paintable,
+				     GdkRegion    *region,
+				     gboolean    (*child_func) (GdkWindow *, gpointer),
+				     gpointer      user_data);
+  void (* process_updates)          (GdkPaintable *paintable,
+				     gboolean      update_children);
+};
+
+GType _gdk_paintable_get_type (void) G_GNUC_CONST;
+
 /* Implementation types */
 GType _gdk_window_impl_get_type (void) G_GNUC_CONST;
 GType _gdk_pixmap_impl_get_type (void) G_GNUC_CONST;
