@@ -30,6 +30,8 @@
 
 #include <config.h>
 
+#include <string.h>
+
 #include <gdk/gdkkeysyms.h>
 
 #include "gtkaboutdialog.h"
@@ -46,15 +48,12 @@
 #include "gtkstock.h"
 #include "gtktextview.h"
 #include "gtkvbox.h"
-#include "gtkviewport.h"
 #include "gtkiconfactory.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 
 #include "gtkalias.h"
 
-#include <string.h>
-
 static GdkColor default_link_color = { 0, 0, 0, 0xeeee };
 static GdkColor default_visited_link_color = { 0, 0x5555, 0x1a1a, 0x8b8b };
 
@@ -172,7 +171,7 @@ static GtkAboutDialogActivateLinkFunc activate_url_hook = NULL;
 static gpointer activate_url_hook_data = NULL;
 static GDestroyNotify activate_url_hook_destroy = NULL;
 
-G_DEFINE_TYPE (GtkAboutDialog, gtk_about_dialog, GTK_TYPE_DIALOG);
+G_DEFINE_TYPE (GtkAboutDialog, gtk_about_dialog, GTK_TYPE_DIALOG)
 
 static void
 gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
