@@ -1,5 +1,6 @@
 /* GTK - The GIMP Toolkit
- * gtkprintoperation-unix.c: Print Operation Details for Unix and Unix like platforms
+ * gtkprintoperation-unix.c: Print Operation Details for Unix 
+ *                           and Unix-like platforms
  * Copyright (C) 2006, Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -35,10 +36,12 @@
 
 #include <cairo-pdf.h>
 #include <cairo-ps.h>
+#include "gtkprivate.h"
 #include "gtkprintunixdialog.h"
 #include "gtkpagesetupunixdialog.h"
 #include "gtkprintbackend.h"
 #include "gtkprinter.h"
+#include "gtkprinter-private.h"
 #include "gtkprintjob.h"
 #include "gtklabel.h"
 #include "gtkalias.h"
@@ -50,7 +53,7 @@ typedef struct
   GMainLoop *loop;
   gboolean data_sent;
 
-  /* Real printing (not preview: */
+  /* Real printing (not preview) */
   GtkPrintJob *job;         /* the job we are sending to the printer */
   cairo_surface_t *surface;
   gulong job_status_changed_tag;
@@ -162,8 +165,37 @@ shell_command_substitute_file (const gchar *cmd,
   return result;
 }
 
+static void
+gtk_print_operation_unix_initialize (void)
+{
+  static gboolean initialized = FALSE;
+
+  if (!initialized)
+    {
+      /**
+       * GtkSettings::gtk-print-preview-command:
+       *
+       * A command to run for displaying the print preview. The command
+       * should contain a %f placeholder, which will get replaced by
+       * the path to the pdf file.
+       *
+       * The preview application is responsible for removing the pdf file
+       * when it is done.
+       *
+       * Since: 2.10
+       */
+      gtk_settings_install_property (g_param_spec_string ("gtk-print-preview-command",
+							  P_("Default command to run when displaying a print preview"),
+							  P_("Command to run when displaying a print preview"),
+							  GTK_PRINT_PREVIEW_COMMAND,
+							  GTK_PARAM_READWRITE)); 
+      initialized = TRUE;
+    }
+}
+
 void
 _gtk_print_operation_platform_backend_launch_preview (GtkPrintOperation *op,
+						      cairo_surface_t   *surface,
 						      GtkWindow         *parent,
 						      const gchar       *filename)
 {
@@ -175,6 +207,10 @@ _gtk_print_operation_platform_backend_launch_preview (GtkPrintOperation *op,
   gchar *quoted_filename;
   GdkScreen *screen;
   GError *error = NULL;
+
+  gtk_print_operation_unix_initialize ();
+
+  cairo_surface_destroy (surface);
  
   settings = gtk_settings_get_default ();
   g_object_get (settings, "gtk-print-preview-command", &preview_cmd, NULL);
@@ -270,8 +306,7 @@ unix_end_run (GtkPrintOperation *op,
   if (op_unix->job != NULL)
     gtk_print_job_send (op_unix->job,
                         unix_finish_send, 
-                        op_unix, NULL,
-		        NULL);
+                        op_unix, NULL);
 
   if (wait)
     {
@@ -309,7 +344,10 @@ get_print_dialog (GtkPrintOperation *op,
 						 GTK_PRINT_CAPABILITY_COPIES |
 						 GTK_PRINT_CAPABILITY_COLLATE |
 						 GTK_PRINT_CAPABILITY_REVERSE |
-						 GTK_PRINT_CAPABILITY_SCALE);
+						 GTK_PRINT_CAPABILITY_SCALE |
+                                                 GTK_PRINT_CAPABILITY_GENERATE_PDF |
+						 GTK_PRINT_CAPABILITY_GENERATE_PS |
+                                                 GTK_PRINT_CAPABILITY_PREVIEW);
 
   if (priv->print_settings)
     gtk_print_unix_dialog_set_settings (GTK_PRINT_UNIX_DIALOG (pd),
@@ -350,7 +388,7 @@ typedef struct
 {
   GtkPrintOperation           *op;
   gboolean                     do_print;
-  GError                     **error;
+  gboolean                     do_preview;
   GtkPrintOperationResult      result;
   GtkPrintOperationPrintFunc   print_cb;
   GDestroyNotify               destroy;
@@ -376,9 +414,6 @@ finish_print (PrintResponseData *rdata,
   GtkPrintOperation *op = rdata->op;
   GtkPrintOperationPrivate *priv = op->priv;
   GtkPrintJob *job;
-  gboolean is_preview;
-
-  is_preview = rdata->result == GTK_PRINT_OPERATION_RESULT_PREVIEW;
   
   if (rdata->do_print)
     {
@@ -386,12 +421,12 @@ finish_print (PrintResponseData *rdata,
       priv->print_context = _gtk_print_context_new (op);
       _gtk_print_context_set_page_setup (priv->print_context, page_setup);
 
-      if (!is_preview)
+      if (!rdata->do_preview)
         {
 	  GtkPrintOperationUnix *op_unix;
 	  cairo_t *cr;
 	  
-	  op_unix = g_new0 (GtkPrintOperationUnix, 1);     
+	  op_unix = g_new0 (GtkPrintOperationUnix, 1);
 	  priv->platform_data = op_unix;
 	  priv->free_platform_data = (GDestroyNotify) op_unix_free;
 	  op_unix->parent = rdata->parent;
@@ -404,9 +439,10 @@ finish_print (PrintResponseData *rdata,
           op_unix->job = job;
           gtk_print_job_set_track_print_status (job, priv->track_print_status);
 	  
-	  op_unix->surface = gtk_print_job_get_surface (job, rdata->error);
+	  op_unix->surface = gtk_print_job_get_surface (job, &priv->error);
 	  if (op_unix->surface == NULL) 
             {
+	      rdata->result = GTK_PRINT_OPERATION_RESULT_ERROR;
 	      rdata->do_print = FALSE;
 	      goto out;
             }
@@ -435,12 +471,7 @@ finish_print (PrintResponseData *rdata,
     } 
  out:
   if (rdata->print_cb)
-    {
-      if (rdata->do_print)
-        rdata->print_cb (op, rdata->parent, is_preview); 
-      else
-       _gtk_print_operation_set_status (op, GTK_PRINT_STATUS_FINISHED_ABORTED, NULL); 
-    }
+    rdata->print_cb (op, rdata->parent, rdata->do_print, rdata->result); 
 
   if (rdata->destroy)
     rdata->destroy (rdata);
@@ -459,17 +490,21 @@ handle_print_response (GtkWidget *dialog,
 
   if (response == GTK_RESPONSE_OK)
     {
-      rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
-
       printer = gtk_print_unix_dialog_get_selected_printer (GTK_PRINT_UNIX_DIALOG (pd));
+
+      rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
+      rdata->do_preview = FALSE;
       if (printer != NULL)
 	rdata->do_print = TRUE;
     } 
   else if (response == GTK_RESPONSE_APPLY)
     {
       /* print preview */
-      rdata->result = GTK_PRINT_OPERATION_RESULT_PREVIEW;
+      rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
+      rdata->do_preview = TRUE;
       rdata->do_print = TRUE;
+
+      rdata->op->priv->action = GTK_PRINT_OPERATION_ACTION_PREVIEW;
     }
 
   if (rdata->do_print)
@@ -504,10 +539,7 @@ found_printer (GtkPrinter        *printer,
 
   if (printer != NULL) 
     {
-      if (priv->show_preview)
-        rdata->result = GTK_PRINT_OPERATION_RESULT_PREVIEW;
-      else
-        rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
+      rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
 
       rdata->do_print = TRUE;
 
@@ -536,6 +568,7 @@ found_printer (GtkPrinter        *printer,
 
 void
 _gtk_print_operation_platform_backend_run_dialog_async (GtkPrintOperation          *op,
+							gboolean                    show_dialog,
                                                         GtkWindow                  *parent,
 							GtkPrintOperationPrintFunc  print_cb)
 {
@@ -547,13 +580,12 @@ _gtk_print_operation_platform_backend_run_dialog_async (GtkPrintOperation
   rdata->op = g_object_ref (op);
   rdata->do_print = FALSE;
   rdata->result = GTK_PRINT_OPERATION_RESULT_CANCEL;
-  rdata->error = NULL;
   rdata->print_cb = print_cb;
   rdata->parent = parent;
   rdata->loop = NULL;
   rdata->destroy = print_response_data_free;
   
-  if (op->priv->show_dialog)
+  if (show_dialog)
     {
       pd = get_print_dialog (op, parent);
       gtk_window_set_modal (GTK_WINDOW (pd), TRUE);
@@ -573,22 +605,86 @@ _gtk_print_operation_platform_backend_run_dialog_async (GtkPrintOperation
     }
 }
 
+static cairo_status_t
+write_preview (void                *closure,
+               const unsigned char *data,
+               unsigned int         length)
+{
+  gint fd = GPOINTER_TO_INT (closure);
+  gssize written;
+  
+  while (length > 0) 
+    {
+      written = write (fd, data, length);
+
+      if (written == -1)
+	{
+	  if (errno == EAGAIN || errno == EINTR)
+	    continue;
+	  
+	  return CAIRO_STATUS_WRITE_ERROR;
+	}    
+
+      data += written;
+      length -= written;
+    }
+
+  return CAIRO_STATUS_SUCCESS;
+}
+
+static void
+close_preview (void *data)
+{
+  gint fd = GPOINTER_TO_INT (data);
+
+  close (fd);
+}
+
 cairo_surface_t *
 _gtk_print_operation_platform_backend_create_preview_surface (GtkPrintOperation *op,
 							      GtkPageSetup      *page_setup,
 							      gdouble           *dpi_x,
 							      gdouble           *dpi_y,
-							      const gchar       *target)
+							      gchar            **target)
 {
+  gchar *filename;
+  gint fd;
   GtkPaperSize *paper_size;
   gdouble w, h;
+  cairo_surface_t *surface;
+  static cairo_user_data_key_t key;
   
+  filename = g_build_filename (g_get_tmp_dir (), "previewXXXXXX.pdf", NULL);
+  fd = g_mkstemp (filename);
+  *target = filename;
+  
+  g_print ("target is %s\n", filename);
+
   paper_size = gtk_page_setup_get_paper_size (page_setup);
   w = gtk_paper_size_get_width (paper_size, GTK_UNIT_POINTS);
   h = gtk_paper_size_get_height (paper_size, GTK_UNIT_POINTS);
     
   *dpi_x = *dpi_y = 72;
-  return cairo_pdf_surface_create (target, w, h);
+  surface = cairo_pdf_surface_create_for_stream (write_preview, GINT_TO_POINTER(fd), w, h);
+ 
+  cairo_surface_set_user_data (surface, &key, GINT_TO_POINTER (fd), close_preview);
+
+  return surface;
+}
+
+void
+_gtk_print_operation_platform_backend_preview_start_page (GtkPrintOperation *op,
+							  cairo_surface_t   *surface,
+							  cairo_t           *cr)
+{
+}
+
+void
+_gtk_print_operation_platform_backend_preview_end_page (GtkPrintOperation *op,
+							cairo_surface_t   *surface,
+							cairo_t           *cr)
+{
+  cairo_show_page (cr);
 }
 
 void
@@ -608,9 +704,9 @@ _gtk_print_operation_platform_backend_resize_preview_surface (GtkPrintOperation
 
 GtkPrintOperationResult
 _gtk_print_operation_platform_backend_run_dialog (GtkPrintOperation *op,
+						  gboolean           show_dialog,
 						  GtkWindow         *parent,
-						  gboolean          *do_print,
-						  GError           **error)
+						  gboolean          *do_print)
  {
   GtkWidget *pd;
   PrintResponseData rdata;
@@ -620,13 +716,12 @@ _gtk_print_operation_platform_backend_run_dialog (GtkPrintOperation *op,
   rdata.op = op;
   rdata.do_print = FALSE;
   rdata.result = GTK_PRINT_OPERATION_RESULT_CANCEL;
-  rdata.error = error;
   rdata.print_cb = NULL;
   rdata.destroy = NULL;
   rdata.parent = parent;
   rdata.loop = NULL;
 
-  if (op->priv->show_dialog)
+  if (show_dialog)
     {
       pd = get_print_dialog (op, parent);
 
@@ -670,7 +765,9 @@ page_setup_data_free (gpointer data)
 {
   PageSetupResponseData *rdata = data;
 
-  g_object_unref (rdata->page_setup);
+  if (rdata->page_setup)
+    g_object_unref (rdata->page_setup);
+
   g_free (rdata);
 }
 
