@@ -556,8 +556,8 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkPrintOperationClass, done),
 		  NULL, NULL,
-		  g_cclosure_marshal_VOID__INT,
-		  G_TYPE_NONE, 1, G_TYPE_INT);
+		  g_cclosure_marshal_VOID__ENUM,
+		  G_TYPE_NONE, 1, GTK_TYPE_PRINT_OPERATION_RESULT);
 
   /**
    * GtkPrintOperation::begin-print:
@@ -583,7 +583,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 		  G_TYPE_NONE, 1, GTK_TYPE_PRINT_CONTEXT);
 
    /**
-   * Gtkprintoperation::paginate:
+   * GtkPrintOperation::paginate:
    * @operation: the #GtkPrintOperation on which the signal was emitted
    * @context: the #GtkPrintContext for the current operation
    *
@@ -600,6 +600,8 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * it all in the begin-print handler, and set the number of pages
    * from there.
    *
+   * Return value: %TRUE if pagination is complete
+   *
    * Since: 2.10
    */
   signals[PAGINATE] =
@@ -1013,10 +1015,16 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * GtkPrintOperation:allow-async:
    *
    * Determines whether the print operation may run asynchronously or not.
+   *
    * Some systems don't support asynchronous printing, but those that do
    * will return %GTK_PRINT_OPERATION_RESULT_IN_PROGRESS as the status, and
    * emit the done signal when the operation is actually done.
    *
+   * The Windows port does not support asynchronous operation
+   * at all (this is unlikely to change). On other platforms, all actions
+   * except for %GTK_PRINT_OPERATION_ACTION_EXPORT support asynchronous
+   * operation.
+   *
    * Since: 2.10
    */
   g_object_class_install_property (gobject_class,
@@ -1630,7 +1638,7 @@ gtk_print_operation_set_allow_async (GtkPrintOperation  *op,
 
 
 /**
- * gtk_print_operation_set_custom_tag_label:
+ * gtk_print_operation_set_custom_tab_label:
  * @op: a #GtkPrintOperation
  * @label: the label to use, or %NULL to use the default label
  *
@@ -1840,10 +1848,9 @@ typedef struct
 
   gint page, start, end, inc;
 
-  gboolean initialized;
-
   GtkWidget *progress;
  
+  gboolean initialized;
   gboolean is_preview; 
 } PrintPagesData;
 
@@ -2194,7 +2201,7 @@ print_pages (GtkPrintOperation       *op,
 
   data = g_new0 (PrintPagesData, 1);
   data->op = g_object_ref (op);
-  data->is_preview = (op->priv->action == GTK_PRINT_OPERATION_ACTION_PREVIEW);
+  data->is_preview = (priv->action == GTK_PRINT_OPERATION_ACTION_PREVIEW);
 
   if (priv->show_progress)
     {
@@ -2221,7 +2228,7 @@ print_pages (GtkPrintOperation       *op,
       
       g_signal_emit_by_name (op, "preview",
 			     GTK_PRINT_OPERATION_PREVIEW (op),
-			     op->priv->print_context,
+			     priv->print_context,
 			     parent,
 			     &handled);
       
@@ -2382,6 +2389,9 @@ gtk_print_operation_run (GtkPrintOperation        *op,
   
   if (action == GTK_PRINT_OPERATION_ACTION_EXPORT)
     {
+      /* note: if you implement async EXPORT, update the docs
+       * docs for the allow-async property.
+       */
       priv->is_sync = TRUE;
       g_return_val_if_fail (priv->export_filename != NULL, GTK_PRINT_OPERATION_RESULT_ERROR);
       result = run_pdf (op, parent, &do_print);
