@@ -39,16 +39,15 @@
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtksettings.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkalias.h"
 
-#define MIN_SPIN_BUTTON_WIDTH              30
-#define SPIN_BUTTON_INITIAL_TIMER_DELAY    200
-#define SPIN_BUTTON_TIMER_DELAY            20
-#define MAX_TIMER_CALLS                    5
-#define EPSILON                            1e-10
-#define	MAX_DIGITS			   20
-#define MIN_ARROW_WIDTH			   6
+#define MIN_SPIN_BUTTON_WIDTH 30
+#define MAX_TIMER_CALLS       5
+#define EPSILON               1e-10
+#define	MAX_DIGITS            20
+#define MIN_ARROW_WIDTH       6
 
 enum {
   PROP_0,
@@ -69,12 +68,11 @@ enum
   OUTPUT,
   VALUE_CHANGED,
   CHANGE_VALUE,
+  WRAPPED,
   LAST_SIGNAL
 };
 
-static void gtk_spin_button_class_init     (GtkSpinButtonClass *klass);
 static void gtk_spin_button_editable_init  (GtkEditableClass   *iface);
-static void gtk_spin_button_init           (GtkSpinButton      *spin_button);
 static void gtk_spin_button_finalize       (GObject            *object);
 static void gtk_spin_button_destroy        (GtkObject          *object);
 static void gtk_spin_button_set_property   (GObject         *object,
@@ -142,48 +140,13 @@ static gint spin_button_get_shadow_type    (GtkSpinButton      *spin_button);
 static void spin_button_redraw             (GtkSpinButton      *spin_button);
 
 
-static GtkEntryClass *parent_class = NULL;
 static guint spinbutton_signals[LAST_SIGNAL] = {0};
 
 #define NO_ARROW 2
 
-GType
-gtk_spin_button_get_type (void)
-{
-  static GType spin_button_type = 0;
-
-  if (!spin_button_type)
-    {
-      static const GTypeInfo spin_button_info =
-      {
-	sizeof (GtkSpinButtonClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_spin_button_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkSpinButton),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_spin_button_init,
-      };
-
-      static const GInterfaceInfo editable_info =
-      {
-	(GInterfaceInitFunc) gtk_spin_button_editable_init, /* interface_init */
-	NULL, /* interface_finalize */
-	NULL  /* interface_data */
-      };
-
-      spin_button_type =
-	g_type_register_static (GTK_TYPE_ENTRY, "GtkSpinButton",
-				&spin_button_info, 0);
-
-      g_type_add_interface_static (spin_button_type,
-				   GTK_TYPE_EDITABLE,
-				   &editable_info);
-    }
-  return spin_button_type;
-}
+G_DEFINE_TYPE_WITH_CODE (GtkSpinButton, gtk_spin_button, GTK_TYPE_ENTRY,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_EDITABLE,
+						gtk_spin_button_editable_init))
 
 #define add_spin_binding(binding_set, keyval, mask, scroll)            \
   gtk_binding_entry_add_signal (binding_set, keyval, mask,             \
@@ -203,8 +166,6 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
   widget_class   = (GtkWidgetClass*)   class;
   entry_class    = (GtkEntryClass*)    class;
 
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class->finalize = gtk_spin_button_finalize;
 
   gobject_class->set_property = gtk_spin_button_set_property;
@@ -242,17 +203,17 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
                                                         P_("Adjustment"),
                                                         P_("The adjustment that holds the value of the spinbutton"),
                                                         GTK_TYPE_ADJUSTMENT,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_CLIMB_RATE,
-                                   g_param_spec_double ("climb_rate",
+                                   g_param_spec_double ("climb-rate",
 							P_("Climb Rate"),
 							P_("The acceleration rate when you hold down a button"),
 							0.0,
 							G_MAXDOUBLE,
 							0.0,
-							G_PARAM_READWRITE));  
+							GTK_PARAM_READWRITE));  
   
   g_object_class_install_property (gobject_class,
                                    PROP_DIGITS,
@@ -262,15 +223,15 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 						      0,
 						      MAX_DIGITS,
 						      0,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_SNAP_TO_TICKS,
-                                   g_param_spec_boolean ("snap_to_ticks",
+                                   g_param_spec_boolean ("snap-to-ticks",
 							 P_("Snap to Ticks"),
 							 P_("Whether erroneous values are automatically changed to a spin button's nearest step increment"),
 							 FALSE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_NUMERIC,
@@ -278,7 +239,7 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 							 P_("Numeric"),
 							 P_("Whether non-numeric characters should be ignored"),
 							 FALSE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_WRAP,
@@ -286,16 +247,16 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 							 P_("Wrap"),
 							 P_("Whether a spin button should wrap upon reaching its limits"),
 							 FALSE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_UPDATE_POLICY,
-                                   g_param_spec_enum ("update_policy",
+                                   g_param_spec_enum ("update-policy",
 						      P_("Update Policy"),
 						      P_("Whether the spin button should update always, or only when the value is legal"),
 						      GTK_TYPE_SPIN_BUTTON_UPDATE_POLICY,
 						      GTK_UPDATE_ALWAYS,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_VALUE,
@@ -305,18 +266,18 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 							-G_MAXDOUBLE,
 							G_MAXDOUBLE,
 							0.0,
-							G_PARAM_READWRITE));  
+							GTK_PARAM_READWRITE));  
   
   gtk_widget_class_install_style_property_parser (widget_class,
-						  g_param_spec_enum ("shadow_type", 
+						  g_param_spec_enum ("shadow-type", 
 								     "Shadow Type", 
 								     P_("Style of bevel around the spin button"),
 								     GTK_TYPE_SHADOW_TYPE,
 								     GTK_SHADOW_IN,
-								     G_PARAM_READABLE),
+								     GTK_PARAM_READABLE),
 						  gtk_rc_property_parse_enum);
   spinbutton_signals[INPUT] =
-    g_signal_new ("input",
+    g_signal_new (I_("input"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkSpinButtonClass, input),
@@ -326,7 +287,7 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 		  G_TYPE_POINTER);
 
   spinbutton_signals[OUTPUT] =
-    g_signal_new ("output",
+    g_signal_new (I_("output"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkSpinButtonClass, output),
@@ -335,7 +296,7 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 		  G_TYPE_BOOLEAN, 0);
 
   spinbutton_signals[VALUE_CHANGED] =
-    g_signal_new ("value_changed",
+    g_signal_new (I_("value_changed"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkSpinButtonClass, value_changed),
@@ -343,9 +304,27 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
 
+  /**
+   * GtkSpinButton::wrapped:
+   * @spinbutton: the object which received the signal
+   *
+   * The wrapped signal is emitted right after the spinbutton wraps
+   * from its maximum to minimum value or vice-versa.
+   *
+   * Since: 2.10
+   */
+  spinbutton_signals[WRAPPED] =
+    g_signal_new (I_("wrapped"),
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GtkSpinButtonClass, wrapped),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
+
   /* Action signals */
   spinbutton_signals[CHANGE_VALUE] =
-    g_signal_new ("change_value",
+    g_signal_new (I_("change_value"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                   G_STRUCT_OFFSET (GtkSpinButtonClass, change_value),
@@ -494,7 +473,7 @@ gtk_spin_button_finalize (GObject *object)
 {
   gtk_spin_button_set_adjustment (GTK_SPIN_BUTTON (object), NULL);
   
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_spin_button_parent_class)->finalize (object);
 }
 
 static void
@@ -502,7 +481,7 @@ gtk_spin_button_destroy (GtkObject *object)
 {
   gtk_spin_button_stop_spinning (GTK_SPIN_BUTTON (object));
   
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+  GTK_OBJECT_CLASS (gtk_spin_button_parent_class)->destroy (object);
 }
 
 static void
@@ -510,7 +489,7 @@ gtk_spin_button_map (GtkWidget *widget)
 {
   if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_MAPPED (widget))
     {
-      GTK_WIDGET_CLASS (parent_class)->map (widget);
+      GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->map (widget);
       gdk_window_show (GTK_SPIN_BUTTON (widget)->panel);
     }
 }
@@ -523,7 +502,7 @@ gtk_spin_button_unmap (GtkWidget *widget)
       gtk_spin_button_stop_spinning (GTK_SPIN_BUTTON (widget));
 
       gdk_window_hide (GTK_SPIN_BUTTON (widget)->panel);
-      GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+      GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->unmap (widget);
     }
 }
 
@@ -544,7 +523,7 @@ gtk_spin_button_realize (GtkWidget *widget)
   widget->allocation.width -= arrow_size + 2 * widget->style->xthickness;
   gtk_widget_set_events (widget, gtk_widget_get_events (widget) |
 			 GDK_KEY_RELEASE_MASK);
-  GTK_WIDGET_CLASS (parent_class)->realize (widget);
+  GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->realize (widget);
 
   widget->allocation.width = real_width;
   
@@ -586,7 +565,7 @@ gtk_spin_button_unrealize (GtkWidget *widget)
 {
   GtkSpinButton *spin = GTK_SPIN_BUTTON (widget);
 
-  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+  GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->unrealize (widget);
 
   if (spin->panel)
     {
@@ -631,7 +610,7 @@ gtk_spin_button_size_request (GtkWidget      *widget,
   spin_button = GTK_SPIN_BUTTON (widget);
   arrow_size = spin_button_get_arrow_size (spin_button);
   
-  GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+  GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->size_request (widget, requisition);
 
   if (entry->width_chars < 0)
     {
@@ -725,7 +704,7 @@ gtk_spin_button_size_allocate (GtkWidget     *widget,
   panel_allocation.y = allocation->y + (allocation->height -
 				       panel_allocation.height) / 2;
 
-  GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, &entry_allocation);
+  GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->size_allocate (widget, &entry_allocation);
 
   if (GTK_WIDGET_REALIZED (widget))
     {
@@ -756,7 +735,7 @@ gtk_spin_button_expose (GtkWidget      *widget,
       GdkRectangle rect;
 
       if (event->window != spin->panel)
-	GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+	GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->expose_event (widget, event);
 
       /* we redraw the panel even if it wasn't exposed. This is
        * because spin->panel is not a child window of widget->window,
@@ -965,7 +944,7 @@ gtk_spin_button_focus_out (GtkWidget     *widget,
   if (GTK_ENTRY (widget)->editable)
     gtk_spin_button_update (GTK_SPIN_BUTTON (widget));
 
-  return GTK_WIDGET_CLASS (parent_class)->focus_out_event (widget, event);
+  return GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->focus_out_event (widget, event);
 }
 
 static void
@@ -1049,10 +1028,15 @@ start_spinning (GtkSpinButton *spin,
   
   if (!spin->timer)
     {
+      GtkSettings *settings = gtk_widget_get_settings (GTK_WIDGET (spin));
+      guint        timeout;
+
+      g_object_get (settings, "gtk-timeout-initial", &timeout, NULL);
+
       spin->timer_step = step;
       spin->need_timer = TRUE;
-      spin->timer = g_timeout_add (SPIN_BUTTON_INITIAL_TIMER_DELAY, 
-				   (GSourceFunc) gtk_spin_button_timer, 
+      spin->timer = g_timeout_add (timeout,
+				   (GSourceFunc) gtk_spin_button_timer,
 				   (gpointer) spin);
     }
   gtk_spin_button_real_spin (spin, click_child == GTK_ARROW_UP ? step : -step);
@@ -1098,7 +1082,7 @@ gtk_spin_button_button_press (GtkWidget      *widget,
 	  return TRUE;
 	}
       else
-	return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
+	return GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->button_press_event (widget, event);
     }
   return FALSE;
 }
@@ -1149,7 +1133,7 @@ gtk_spin_button_button_release (GtkWidget      *widget,
       return TRUE;
     }
   else
-    return GTK_WIDGET_CLASS (parent_class)->button_release_event (widget, event);
+    return GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->button_release_event (widget, event);
 }
 
 static gint
@@ -1183,7 +1167,7 @@ gtk_spin_button_motion_notify (GtkWidget      *widget,
       return FALSE;
     }
 	  
-  return GTK_WIDGET_CLASS (parent_class)->motion_notify_event (widget, event);
+  return GTK_WIDGET_CLASS (gtk_spin_button_parent_class)->motion_notify_event (widget, event);
 }
 
 static gint
@@ -1202,8 +1186,13 @@ gtk_spin_button_timer (GtkSpinButton *spin_button)
 
       if (spin_button->need_timer)
 	{
+          GtkSettings *settings = gtk_widget_get_settings (GTK_WIDGET (spin_button));
+          guint        timeout;
+
+          g_object_get (settings, "gtk-timeout-repeat", &timeout, NULL);
+
 	  spin_button->need_timer = FALSE;
-	  spin_button->timer = g_timeout_add (SPIN_BUTTON_TIMER_DELAY, 
+	  spin_button->timer = g_timeout_add (timeout,
 					      (GSourceFunc) gtk_spin_button_timer, 
 					      (gpointer) spin_button);
 	}
@@ -1380,7 +1369,7 @@ gtk_spin_button_activate (GtkEntry *entry)
     gtk_spin_button_update (GTK_SPIN_BUTTON (entry));
 
   /* Chain up so that entry->activates_default is honored */
-  parent_class->activate (entry);
+  GTK_ENTRY_CLASS (gtk_spin_button_parent_class)->activate (entry);
 }
 
 static void
@@ -1391,7 +1380,7 @@ gtk_spin_button_insert_text (GtkEditable *editable,
 {
   GtkEntry *entry = GTK_ENTRY (editable);
   GtkSpinButton *spin = GTK_SPIN_BUTTON (editable);
-  GtkEditableClass *parent_editable_iface = g_type_interface_peek (parent_class, GTK_TYPE_EDITABLE);
+  GtkEditableClass *parent_editable_iface = g_type_interface_peek (gtk_spin_button_parent_class, GTK_TYPE_EDITABLE);
  
   if (spin->numeric)
     {
@@ -1471,6 +1460,7 @@ gtk_spin_button_real_spin (GtkSpinButton *spin_button,
 {
   GtkAdjustment *adj;
   gdouble new_value = 0.0;
+  gboolean wrapped = FALSE;
   
   adj = spin_button->adjustment;
 
@@ -1481,7 +1471,10 @@ gtk_spin_button_real_spin (GtkSpinButton *spin_button,
       if (spin_button->wrap)
 	{
 	  if (fabs (adj->value - adj->upper) < EPSILON)
-	    new_value = adj->lower;
+	    {
+	      new_value = adj->lower;
+	      wrapped = TRUE;
+	    }
 	  else if (new_value > adj->upper)
 	    new_value = adj->upper;
 	}
@@ -1493,7 +1486,10 @@ gtk_spin_button_real_spin (GtkSpinButton *spin_button,
       if (spin_button->wrap)
 	{
 	  if (fabs (adj->value - adj->lower) < EPSILON)
-	    new_value = adj->upper;
+	    {
+	      new_value = adj->upper;
+	      wrapped = TRUE;
+	    }
 	  else if (new_value < adj->lower)
 	    new_value = adj->lower;
 	}
@@ -1504,6 +1500,9 @@ gtk_spin_button_real_spin (GtkSpinButton *spin_button,
   if (fabs (new_value - adj->value) > EPSILON)
     gtk_adjustment_set_value (adj, new_value);
 
+  if (wrapped)
+    g_signal_emit (spin_button, spinbutton_signals[WRAPPED], 0);
+
   spin_button_redraw (spin_button);
 }
 
@@ -1596,11 +1595,11 @@ gtk_spin_button_new (GtkAdjustment *adjustment,
  * #GtkSpinButton without manually creating an adjustment. The value is 
  * initially set to the minimum value and a page increment of 10 * @step
  * is the default. The precision of the spin button is equivalent to the 
- * precision of @step.
- *
+ * precision of @step. 
+ * 
  * Note that the way in which the precision is derived works best if @step 
  * is a power of ten. If the resulting precision is not suitable for your 
- * needs, use gtk_spin_button_set_digits() to correct it. 
+ * needs, use gtk_spin_button_set_digits() to correct it.
  * 
  * Return value: The new spin button as a #GtkWidget.
  **/
@@ -1677,8 +1676,7 @@ gtk_spin_button_set_adjustment (GtkSpinButton *spin_button,
       spin_button->adjustment = adjustment;
       if (adjustment)
         {
-	  g_object_ref (adjustment);
-	  gtk_object_sink (GTK_OBJECT (adjustment));
+	  g_object_ref_sink (adjustment);
 	  g_signal_connect (adjustment, "value_changed",
 			    G_CALLBACK (gtk_spin_button_value_changed),
 			    spin_button);
@@ -1696,7 +1694,7 @@ gtk_spin_button_set_adjustment (GtkSpinButton *spin_button,
 
 /**
  * gtk_spin_button_get_adjustment:
- * @spin_button: 
+ * @spin_button: a #GtkSpinButton
  * 
  * Get the adjustment associated with a #GtkSpinButton
  * 
@@ -2052,7 +2050,7 @@ spin_button_get_shadow_type (GtkSpinButton *spin_button)
 {
   GtkShadowType rc_shadow_type;
 
-  gtk_widget_style_get (GTK_WIDGET (spin_button), "shadow_type", &rc_shadow_type, NULL);
+  gtk_widget_style_get (GTK_WIDGET (spin_button), "shadow-type", &rc_shadow_type, NULL);
 
   return rc_shadow_type;
 }
