@@ -74,6 +74,8 @@ static void gtk_tool_button_toolbar_reconfigured (GtkToolItem *tool_item);
 static gboolean   gtk_tool_button_create_menu_proxy (GtkToolItem     *item);
 static void       button_clicked                    (GtkWidget       *widget,
 						     GtkToolButton   *button);
+static void gtk_tool_button_style_set      (GtkWidget          *widget,
+					    GtkStyle           *prev_style);
 
 static void gtk_tool_button_construct_contents (GtkToolItem *tool_item);
       
@@ -101,24 +103,13 @@ gtk_tool_button_get_type (void)
   static GtkType type = 0;
 
   if (!type)
-    {
-      static const GTypeInfo type_info =
-	{
-	  sizeof (GtkToolButtonClass),
-	  (GBaseInitFunc) NULL,
-	  (GBaseFinalizeFunc) NULL,
-	  (GClassInitFunc) gtk_tool_button_class_init,
-	  (GClassFinalizeFunc) NULL,
-	  NULL,
-	  sizeof (GtkToolButton),
-	  0, /* n_preallocs */
-	  (GInstanceInitFunc) gtk_tool_button_init,
-	};
-
-      type = g_type_register_static (GTK_TYPE_TOOL_ITEM,
-				     "GtkToolButton",
-				     &type_info, 0);
-    }
+    type = g_type_register_static_simple (GTK_TYPE_TOOL_ITEM,
+					  I_("GtkToolButton"),
+					  sizeof (GtkToolButtonClass),
+					  (GClassInitFunc) gtk_tool_button_class_init,
+					  sizeof (GtkToolButton),
+					  (GInstanceInitFunc) gtk_tool_button_init,
+					  0);
   return type;
 }
 
@@ -140,6 +131,8 @@ gtk_tool_button_class_init (GtkToolButtonClass *klass)
   object_class->notify = gtk_tool_button_property_notify;
   object_class->finalize = gtk_tool_button_finalize;
 
+  widget_class->style_set = gtk_tool_button_style_set;
+
   tool_item_class->create_menu_proxy = gtk_tool_button_create_menu_proxy;
   tool_item_class->toolbar_reconfigured = gtk_tool_button_toolbar_reconfigured;
   
@@ -234,6 +227,22 @@ gtk_tool_button_class_init (GtkToolButtonClass *klass)
 							GTK_TYPE_WIDGET,
 							GTK_PARAM_READWRITE));
 
+  /**
+   * GtkButton::icon-spacing:
+   * 
+   * Spacing in pixels between the icon and label.
+   * 
+   * Since: 2.10
+   */
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("icon-spacing",
+							     P_("Icon spacing"),
+							     P_("Spacing in pixels between the icon and label"),
+							     0,
+							     G_MAXINT,
+							     0,
+							     GTK_PARAM_READWRITE));
+
 /**
  * GtkToolButton::clicked:
  * @toolbutton: the object that emitted the signal
@@ -242,9 +251,9 @@ gtk_tool_button_class_init (GtkToolButtonClass *klass)
  * or activated with the keyboard.
  **/
   toolbutton_signals[CLICKED] =
-    g_signal_new ("clicked",
+    g_signal_new (I_("clicked"),
 		  G_OBJECT_CLASS_TYPE (klass),
-		  G_SIGNAL_RUN_FIRST,
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkToolButtonClass, clicked),
 		  NULL, NULL,
 		  g_cclosure_marshal_VOID__VOID,
@@ -284,6 +293,11 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
   gboolean need_icon = FALSE;
   GtkIconSize icon_size;
   GtkWidget *box = NULL;
+  guint icon_spacing;
+
+  gtk_widget_style_get (GTK_WIDGET (tool_item), 
+			"icon-spacing", &icon_spacing,
+			NULL);
 
   if (button->priv->icon_widget && button->priv->icon_widget->parent)
     {
@@ -319,6 +333,22 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
     {
       need_label = TRUE;
     }
+  
+  if (style == GTK_TOOLBAR_ICONS && button->priv->icon_widget == NULL &&
+      button->priv->stock_id == NULL && button->priv->icon_name == NULL)
+    {
+      need_label = TRUE;
+      need_icon = FALSE;
+      style = GTK_TOOLBAR_TEXT;
+    }
+
+  if (style == GTK_TOOLBAR_TEXT && button->priv->label_widget == NULL &&
+      button->priv->stock_id == NULL && button->priv->label_text == NULL)
+    {
+      need_label = FALSE;
+      need_icon = TRUE;
+      style = GTK_TOOLBAR_ICONS;
+    }
 
   if (need_label)
     {
@@ -375,7 +405,8 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 			    NULL);
 	    }
 	}
-      else if (button->priv->stock_id)
+      else if (button->priv->stock_id && 
+	       gtk_icon_factory_lookup_default (button->priv->stock_id))
 	{
 	  icon = gtk_image_new_from_stock (button->priv->stock_id, icon_size);
 	  gtk_widget_show (icon);
@@ -395,7 +426,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
       break;
 
     case GTK_TOOLBAR_BOTH:
-      box = gtk_vbox_new (FALSE, 0);
+      box = gtk_vbox_new (FALSE, icon_spacing);
       if (icon)
 	gtk_box_pack_start (GTK_BOX (box), icon, TRUE, TRUE, 0);
       gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
@@ -403,7 +434,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
       break;
 
     case GTK_TOOLBAR_BOTH_HORIZ:
-      box = gtk_hbox_new (FALSE, 0);
+      box = gtk_hbox_new (FALSE, icon_spacing);
       if (icon)
 	gtk_box_pack_start (GTK_BOX (box), icon, label? FALSE : TRUE, TRUE, 0);
       if (label)
@@ -548,12 +579,16 @@ clone_image_menu_size (GtkImage *image, GtkSettings *settings)
 					     &width, &height))
 	{
 	  GdkPixbuf *src_pixbuf, *dest_pixbuf;
+	  GtkWidget *cloned_image;
 
 	  src_pixbuf = gtk_image_get_pixbuf (image);
 	  dest_pixbuf = gdk_pixbuf_scale_simple (src_pixbuf, width, height,
 						 GDK_INTERP_BILINEAR);
 
-	  return gtk_image_new_from_pixbuf (dest_pixbuf);
+	  cloned_image = gtk_image_new_from_pixbuf (dest_pixbuf);
+	  g_object_unref (dest_pixbuf);
+
+	  return cloned_image;
 	}
     }
 
@@ -631,6 +666,29 @@ gtk_tool_button_toolbar_reconfigured (GtkToolItem *tool_item)
   gtk_tool_button_construct_contents (tool_item);
 }
 
+static void 
+gtk_tool_button_update_icon_spacing (GtkToolButton *button)
+{
+  GtkWidget *box;
+  guint spacing;
+
+  box = GTK_BIN (button->priv->button)->child;
+  if (GTK_IS_BOX (box))
+    {
+      gtk_widget_style_get (GTK_WIDGET (button), 
+			    "icon-spacing", &spacing,
+			    NULL);
+      gtk_box_set_spacing (GTK_BOX (box), spacing);      
+    }
+}
+
+static void
+gtk_tool_button_style_set (GtkWidget *widget,
+			   GtkStyle  *prev_style)
+{
+  gtk_tool_button_update_icon_spacing (GTK_TOOL_BUTTON (widget));
+}
+
 /**
  * gtk_tool_button_new_from_stock:
  * @stock_id: the name of the stock item 
@@ -927,8 +985,7 @@ gtk_tool_button_set_icon_widget (GtkToolButton *button,
       
       if (icon_widget)
 	{
-	  g_object_ref (icon_widget);
-	  gtk_object_sink (GTK_OBJECT (icon_widget));
+	  g_object_ref_sink (icon_widget);
 	}
 
       button->priv->icon_widget = icon_widget;
@@ -974,8 +1031,7 @@ gtk_tool_button_set_label_widget (GtkToolButton *button,
       
       if (label_widget)
 	{
-	  g_object_ref (label_widget);
-	  gtk_object_sink (GTK_OBJECT (label_widget));
+	  g_object_ref_sink (label_widget);
 	}
 
       button->priv->label_widget = label_widget;
@@ -1032,5 +1088,6 @@ _gtk_tool_button_get_button (GtkToolButton *button)
   return button->priv->button;
 }
 
+
 #define __GTK_TOOL_BUTTON_C__
 #include "gtkaliasdef.c"
